/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REALCAENPSDDRIVER_H
#define REALCAENPSDDRIVER_H

#include <CAENDigitizer.h>

#include "CAENPsdDef.h"
#include "CAENPsdState.h"

namespace caen_psd {

class CAENPsdDriver;

/**
 * Real implementation
 */
class RealCAENPsdDriver : public CAENPsdState {

public:

	/**
	 * Constructor.
	 *
	 * @param Link to the class which used the state
	 */
	RealCAENPsdDriver(CAENPsdDriver* owner);

	/**
	 * Destructor.
	 * Empty.
	 */
	virtual ~RealCAENPsdDriver();

	/*!
	 * \brief Init command implementation
	 */
	virtual void init();

	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear();

	/*!
	 * \brief Reset command implementation
	 */
	virtual void reset();

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam();

	/*!
	 * \brief Set acquisition mode command implementation
	 */
	virtual void setAcqMode();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos();

private:

	void open();
	void close();

	int32 m_boardHandle;

	CAEN_DGTZ_ErrorCode ret;

	CAEN_DGTZ_BoardInfo_t boardInfo;
	DigitizerParams_t boardParams;
	CAEN_DGTZ_DPP_PSD_Params_t dppParams;
};

}
#endif //REALCAENPSDDRIVER_H
