/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "PerfectCAENQdcDriver.h"

#include <controllers/common/acquisition/mode/DPPAcquisitionCommon.h>
#include "CAENQdcDriver.h"

using namespace std;

namespace caen_qdc {

/*
 * Constructor
 */
PerfectCAENQdcDriver::PerfectCAENQdcDriver(CAENQdcDriver* owner) : CAENQdcState(owner) {

	m_boardHandle = 0;
	ret = CAEN_DGTZ_Success;
}

/*
 * Destructor
 */
PerfectCAENQdcDriver::~PerfectCAENQdcDriver() {
	/* Empty */
}

/*!
 * \brief open command implementation
 */
void PerfectCAENQdcDriver::open() {

}

/*!
 * \brief close command implementation
 */
void PerfectCAENQdcDriver::close() {

}

/*!
 * \brief Init command implementation
 */
void PerfectCAENQdcDriver::init() {

	cout << "Init---------------" << endl;
}

/*!
 * \brief Clear command implementation
 */
void PerfectCAENQdcDriver::clear() {

}

/*!
 * \brief Reset command implementation
 */
void PerfectCAENQdcDriver::reset() {

}

/*!
 * \brief Write Param command implementation
 */
void PerfectCAENQdcDriver::writeParam() {

	cout << "writeParam---------------" << endl;
}

/*!
 * \brief Read Infos command implementation
 */
void PerfectCAENQdcDriver::readInfos() {

	owner()->nbBits = 12;
	owner()->model = 0;
	owner()->channels = 64; // nb of group channels with 8 channels per group
	owner()->serialNumber = 1234567890;
	owner()->modelName = "V1740";
	owner()->rocFirmware = "1234567890";
	owner()->amcFirmware = "1234567890";
}

int32_t PerfectCAENQdcDriver::setChannelGroupMask(uint32_t group, uint32_t channelMask) {

    int32_t address;

    ret = 0;

    if (group > owner()->nbChannels() - 1) {
		cerr << "Error InvalidChannelNumber: " << ret << endl;
    	return ret;
    }

    address = getChannelAddress(CAEN_DGTZ_CHANNEL_GROUP_V1740_BASE_ADDRESS, (uint8_t) group);

    cout << "Address: 0x" << hex << address << dec << " Channel mask: " << channelMask << endl;

    return 0;
}

int32_t PerfectCAENQdcDriver::getChannelAddress(uint32_t base, uint16_t channel) {

	//return base + (0x100 * channel);
    int chShift = 8;
    uint32_t res = base & ~(0xF<<chShift); // set bits [11:8] to zero
    res |= channel << chShift;
    return res;
}

int32_t PerfectCAENQdcDriver::setChannelTriggerThreshold(uint32_t channel, uint32_t Tvalue) {

	int gr;
    int ch;

    ret=0;
    uint32_t addr;

    if (channel > 64)
        return -1;

    gr = channel/8;
    ch = channel % 8;

    addr = 0x1000 + 0x100*gr + 0xD0 + 4*ch;
    ret = 0;
	cout << "Address: 0x" << hex << addr << dec << " Threshold: " << Tvalue << endl;

    return ret;
}

int32_t PerfectCAENQdcDriver::setRecordLength(uint32_t recordLength) {

	ret=0;

    return ret;
}

int32_t PerfectCAENQdcDriver::setNumEvAggregate(uint32_t nEvAggr) {

	ret=0;

    return ret;
}

void PerfectCAENQdcDriver::setAcqMode() {

}

}
