/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/* Generated by Together */

#ifndef CFDDRIVER_H
#define CFDDRIVER_H

#include "drivers/global/Vme.h"

namespace cfd {

/**
 * The MotorQclRioDriver class contains the hardware operation to
 * apply on a motor which controled by a Rio QCL vme card.
 */
class CfdDriver: public driver::Vme {

	friend class RealCfdDriver;
	friend class PerfectCfdDriver;

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CfdDriver(const std::string& aName);

	/*!
	 * \brief Destructor
	 */
	virtual ~CfdDriver();

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] command the command to apply on the controller
	 */
	virtual void execute(const std::string& aCommand);

	/*!
	 * Properties
	 */
	// Global
	Property<std::string> useScopeStr;
	Property<std::string> useListModeStr;
	Property<int32> scopeChannel;
    Property<int32> dppAcqMode;
	Property<int32> nbChannels;
	Property<int32> channels;

	// CFD parameters
	Property<int32> delay;
	Property<int32> attenuation;

	// Energy filter parameters
	DynamicProperty<int32> channelActive;
	DynamicProperty<std::string> channelActiveStr;
	DynamicProperty<int32> pulsePolarity;
	DynamicProperty<std::string> pulsePolarityStr;
	DynamicProperty<int32> trapezoidRise;
	DynamicProperty<int32> trapezoidFlat;
	DynamicProperty<int32> decayTime;
	DynamicProperty<int32> peaking;
	DynamicProperty<int32> bitShift;

	static const std::string WRITE_PARAMETERS_COMMAND;

private:

	/*!
	 * \brief Method called before changing the property value
	 * \param[in] value the property value
	 * \throws CannotSetValue the value isn't corrected, or property couldn't be changed
	 */
	void refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue);
	void refreshChannelActiveProperty(int32 index, int32 aValue) throw (CannotSetValue);
	void refreshPulsePolarityProperty(int32 index, int32 aValue) throw (CannotSetValue);
	void refreshDppAcqModeProperty(int32 value) throw (CannotSetValue);

};

}
#endif //CFDDRIVER_H
